@@ -14,11 +14,13 @@
 
 #include "mirall/folderman.h"
 #include "mirall/mirallconfigfile.h"
-#include "mirall/owncloudfolder.h"
+#include "mirall/folder.h"
 #include "mirall/syncresult.h"
 #include "mirall/inotify.h"
 #include "mirall/theme.h"
 
+#include <neon/ne_socket.h>
+
 #ifdef Q_OS_MAC
 #include <CoreServices/CoreServices.h>
 #endif
@@ -32,6 +34,8 @@
 
 namespace Mirall {
 
+FolderMan* FolderMan::_instance = 0;
+
 FolderMan::FolderMan(QObject *parent) :
     QObject(parent),
     _syncEnabled( true )
@@ -48,11 +52,20 @@ FolderMan::FolderMan(QObject *parent) :
             this, SIGNAL(folderSyncStateChange(const QString &)));
 }
 
+FolderMan *FolderMan::instance()
+{
+    if(!_instance) {
+        _instance = new FolderMan;
+        ne_sock_init();
+    }
+
+    return _instance;
+}
+
 FolderMan::~FolderMan()
 {
-    foreach (Folder *folder, _folderMap) {
-        delete folder;
-    }
+    qDeleteAll(_folderMap);
+    ne_sock_exit();
 }
 
 Mirall::Folder::Map FolderMan::map()
@@ -60,31 +73,8 @@ Mirall::Folder::Map FolderMan::map()
     return _folderMap;
 }
 
-
-int FolderMan::setupFolders()
-{
-    // setup a handler to look for configuration changes
-#ifdef CHECK_FOR_SETUP_CHANGES
-    _configFolderWatcher = new FolderWatcher( _folderConfigPath );
-    _configFolderWatcher->setEventInterval(20000);
-    connect(_configFolderWatcher, SIGNAL(folderChanged(const QStringList &)),
-            this, SLOT( slotReparseConfiguration()) );
-#endif
-    int cnt = setupKnownFolders();
-
-    return cnt;
-}
-
-void FolderMan::slotReparseConfiguration()
-{
-    setupKnownFolders();
-}
-
 int FolderMan::unloadAllFolders()
 {
-    // first terminate sync jobs.
-    terminateCurrentSync();
-
     int cnt = 0;
 
     // clear the list of existing folders.
@@ -94,10 +84,11 @@ int FolderMan::unloadAllFolders()
         delete _folderMap.take( i.key() );
         cnt++;
     }
+    _currentSyncFolder.clear();
     return cnt;
 }
 
-int FolderMan::setupKnownFolders()
+int FolderMan::setupFolders()
 {
   qDebug() << "* Setup folders from " << _folderConfigPath;
 
@@ -227,7 +218,7 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
         return folder;
     }
 
-    QSettings settings( cfgFile.filePath(), QSettings::IniFormat);
+    QSettings settings( _folderConfigPath + QLatin1Char('/') + escapedAlias, QSettings::IniFormat);
     qDebug() << "    -> file path: " << settings.fileName();
 
     // Check if the filename is equal to the group setting. If not, use the group
@@ -240,49 +231,38 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
 
     settings.beginGroup( escapedAlias ); // read the group with the same name as the file which is the folder alias
 
-    QString path = settings.value(QLatin1String("localpath")).toString();
+    QString path = settings.value(QLatin1String("localPath")).toString();
     QString backend = settings.value(QLatin1String("backend")).toString();
-    QString targetPath = settings.value( QLatin1String("targetPath") ).toString();
+    QString targetPath = settings.value( QLatin1String("targetPath")).toString();
+    bool paused = settings.value( QLatin1String("paused"), false).toBool();
     // QString connection = settings.value( QLatin1String("connection") ).toString();
     QString alias = unescapeAlias( escapedAlias );
 
-    if (!backend.isEmpty()) {
-
-        if( backend == QLatin1String("owncloud") ) {
-            MirallConfigFile cfgFile;
-
-            // assemble the owncloud url to pass to csync, incl. webdav
-            QString oCUrl = cfgFile.ownCloudUrl( QString::null, true );
-
-            // cut off the leading slash, oCUrl always has a trailing.
-            if( targetPath.startsWith(QLatin1Char('/')) ) {
-                targetPath.remove(0,1);
-            }
-
-            folder = new ownCloudFolder( alias, path, oCUrl + targetPath, this );
-            folder->setConfigFile(file);
-        } else {
-            qWarning() << "unknown backend" << backend;
-            return 0;
-        }
+    if (backend.isEmpty() || backend != QLatin1String("owncloud")) {
+        qWarning() << "obsolete configuration of type" << backend;
+        return 0;
     }
 
-    if( folder ) {
-        folder->setBackend( backend );
-        // folder->setOnlyOnlineEnabled(settings.value("folder/onlyOnline", false).toBool());
-        folder->setOnlyThisLANEnabled(settings.value(QLatin1String("folder/onlyThisLAN"), false).toBool());
-
-        _folderMap[alias] = folder;
-
-        qDebug() << "Adding folder to Folder Map " << folder;
-        /* Use a signal mapper to connect the signals to the alias */
-        connect(folder, SIGNAL(scheduleToSync(const QString&)), SLOT(slotScheduleSync(const QString&)));
-        connect(folder, SIGNAL(syncStateChange()), _folderChangeSignalMapper, SLOT(map()));
-        connect(folder, SIGNAL(syncStarted()), SLOT(slotFolderSyncStarted()));
-        connect(folder, SIGNAL(syncFinished(SyncResult)), SLOT(slotFolderSyncFinished(SyncResult)));
-
-        _folderChangeSignalMapper->setMapping( folder, folder->alias() );
+    // cut off the leading slash, oCUrl always has a trailing.
+    if( targetPath.startsWith(QLatin1Char('/')) ) {
+        targetPath.remove(0,1);
     }
+
+    folder = new Folder( alias, path, targetPath, this );
+    folder->setConfigFile(file);
+    qDebug() << "Adding folder to Folder Map " << folder;
+    _folderMap[alias] = folder;
+    if (paused) {
+        _disabledFolders.insert(folder);
+    }
+
+    /* Use a signal mapper to connect the signals to the alias */
+    connect(folder, SIGNAL(scheduleToSync(const QString&)), SLOT(slotScheduleSync(const QString&)));
+    connect(folder, SIGNAL(syncStateChange()), _folderChangeSignalMapper, SLOT(map()));
+    connect(folder, SIGNAL(syncStarted()), SLOT(slotFolderSyncStarted()));
+    connect(folder, SIGNAL(syncFinished(SyncResult)), SLOT(slotFolderSyncFinished(SyncResult)));
+
+    _folderChangeSignalMapper->setMapping( folder, folder->alias() );
     return folder;
 }
 
@@ -296,6 +276,17 @@ void FolderMan::slotEnableFolder( const QString& alias, bool enable )
     Folder *f = _folderMap[alias];
     if( f ) {
         f->setSyncEnabled(enable);
+        f->evaluateSync(QStringList());
+
+        QSettings settings(_folderConfigPath + QLatin1Char('/') + f->configFile(), QSettings::IniFormat);
+        settings.beginGroup(escapeAlias(f->alias()));
+        if (enable) {
+            settings.remove("paused");
+            _disabledFolders.remove(f);
+        } else {
+            settings.setValue("paused", true);
+            _disabledFolders.insert(f);
+        }
     }
 }
 
@@ -310,10 +301,9 @@ void FolderMan::terminateSyncProcess( const QString& alias )
     if( ! folderAlias.isEmpty() ) {
         Folder *f = _folderMap[folderAlias];
         if( f ) {
-            f->slotTerminateSync();
-
+            f->slotTerminateSync(true);
             if(_currentSyncFolder == folderAlias )
-                _currentSyncFolder = QString::null;
+                _currentSyncFolder.clear();
         }
     }
 }
@@ -330,19 +320,21 @@ Folder *FolderMan::folder( const QString& alias )
 
 SyncResult FolderMan::syncResult( const QString& alias )
 {
-    SyncResult res;
     Folder *f = folder( alias );
+    return syncResult(f);
+}
 
-    if( f ) {
-        res = f->syncResult();
-    }
-    return res;
+SyncResult FolderMan::syncResult( Folder *f )
+{
+   return f ? f->syncResult() : SyncResult();
 }
 
 void FolderMan::slotScheduleAllFolders()
 {
     foreach( Folder *f, _folderMap.values() ) {
-        slotScheduleSync( f->alias() );
+        if (f->syncEnabled()) {
+            slotScheduleSync( f->alias() );
+        }
     }
 }
 
@@ -356,23 +348,29 @@ void FolderMan::slotScheduleSync( const QString& alias )
 
     qDebug() << "Schedule folder " << alias << " to sync!";
     if( _currentSyncFolder == alias ) {
-        // the current folder is currently syncing.
+        qDebug() << " the current folder is currently syncing.";
         return;
     }
 
     if( ! _scheduleQueue.contains(alias )) {
-        _scheduleQueue.append(alias);
+        _scheduleQueue.enqueue(alias);
     } else {
         qDebug() << " II> Sync for folder " << alias << " already scheduled, do not enqueue!";
     }
-
     slotScheduleFolderSync();
-
 }
 
 void FolderMan::setSyncEnabled( bool enabled )
 {
+    if (!_syncEnabled && enabled && !_scheduleQueue.isEmpty()) {
+        // We have things in our queue that were waiting the the connection to go back on.
+        QTimer::singleShot(200, this, SLOT(slotScheduleFolderSync()));
+    }
     _syncEnabled = enabled;
+
+    foreach( Folder *f, _folderMap.values() ) {
+        f->setSyncEnabled(enabled && !_disabledFolders.contains(f));
+    }
 }
 
 /*
@@ -394,11 +392,13 @@ void FolderMan::slotScheduleFolderSync()
 
     qDebug() << "XX slotScheduleFolderSync: folderQueue size: " << _scheduleQueue.count();
     if( ! _scheduleQueue.isEmpty() ) {
-        const QString alias = _scheduleQueue.takeFirst();
+        const QString alias = _scheduleQueue.dequeue();
         if( _folderMap.contains( alias ) ) {
             Folder *f = _folderMap[alias];
-            _currentSyncFolder = alias;
-            f->startSync( QStringList() );
+            if( f->syncEnabled() ) {
+                _currentSyncFolder = alias;
+                f->startSync( QStringList() );
+            }
         }
     }
 }
@@ -420,31 +420,37 @@ void FolderMan::slotFolderSyncFinished( const SyncResult& )
     QTimer::singleShot(200, this, SLOT(slotScheduleFolderSync()));
 }
 
-/**
-  * Add a folder definition to the config
-  * Params:
-  * QString backend
-  * QString alias
-  * QString sourceFolder on local machine
-  * QString targetPath on remote
-  * bool    onlyThisLAN, currently unused.
-  */
-void FolderMan::addFolderDefinition( const QString& backend, const QString& alias,
-                                     const QString& sourceFolder, const QString& targetPath,
-                                     bool onlyThisLAN )
+void FolderMan::addFolderDefinition(const QString& alias, const QString& sourceFolder, const QString& targetPath )
 {
     QString escapedAlias = escapeAlias(alias);
     // Create a settings file named after the alias
     QSettings settings( _folderConfigPath + QLatin1Char('/') + escapedAlias, QSettings::IniFormat);
-
-    settings.setValue(QString::fromLatin1("%1/localPath").arg(escapedAlias),   sourceFolder );
-    settings.setValue(QString::fromLatin1("%1/targetPath").arg(escapedAlias),  targetPath );
-    settings.setValue(QString::fromLatin1("%1/backend").arg(escapedAlias),     backend );
-    settings.setValue(QString::fromLatin1("%1/connection").arg(escapedAlias),  Theme::instance()->appName());
-    settings.setValue(QString::fromLatin1("%1/onlyThisLAN").arg(escapedAlias), onlyThisLAN );
+    settings.beginGroup(escapedAlias);
+    settings.setValue(QLatin1String("localPath"),   sourceFolder );
+    settings.setValue(QLatin1String("targetPath"),  targetPath );
+    // for compat reasons
+    settings.setValue(QLatin1String("backend"),     "owncloud" );
+    settings.setValue(QLatin1String("connection"),  Theme::instance()->appName());
     settings.sync();
 }
 
+Folder *FolderMan::folderForPath(const QUrl &path)
+{
+    QString absolutePath = path.toLocalFile();
+    absolutePath.append("/");
+
+    foreach(Folder* folder, map().values())
+    {
+        if(absolutePath.startsWith(folder->path()))
+        {
+            qDebug() << "found folder: " << folder->path() << " for " << absolutePath;
+            return folder;
+        }
+    }
+
+    return 0;
+}
+
 void FolderMan::removeAllFolderDefinitions()
 {
     foreach( Folder *f, _folderMap.values() ) {
@@ -487,7 +493,10 @@ void FolderMan::removeFolder( const QString& alias )
             qDebug() << "Remove folder config file " << file.fileName();
             file.remove();
         }
-        f->deleteLater();
+        // FIXME: this is a temporar dirty fix against a crash happening because
+        // the csync owncloud module still has static components. Activate the
+        // delete once the module is fixed.
+        // f->deleteLater();
     }
 }
 
@@ -533,11 +542,99 @@ bool FolderMan::startFromScratch( const QString& localFolder )
     return false;
 }
 
-void FolderMan::setProxy()
+void FolderMan::setDirtyProxy(bool value)
 {
     foreach( Folder *f, _folderMap.values() ) {
-        f->setProxy();
+        f->setProxyDirty(value);
     }
 }
 
+
+SyncResult FolderMan::accountStatus(const QList<Folder*> &folders)
+{
+    SyncResult overallResult(SyncResult::Undefined);
+
+    foreach ( Folder *folder, folders ) {
+        SyncResult folderResult = folder->syncResult();
+        SyncResult::Status syncStatus = folderResult.status();
+
+        switch( syncStatus ) {
+        case SyncResult::Undefined:
+            if ( overallResult.status() != SyncResult::Error )
+                overallResult.setStatus(SyncResult::Error);
+            break;
+        case SyncResult::NotYetStarted:
+            overallResult.setStatus( SyncResult::NotYetStarted );
+            break;
+        case SyncResult::SyncPrepare:
+            overallResult.setStatus( SyncResult::SyncPrepare );
+            break;
+        case SyncResult::SyncRunning:
+            overallResult.setStatus( SyncResult::SyncRunning );
+            break;
+        case SyncResult::Unavailable:
+            overallResult.setStatus( SyncResult::Unavailable );
+            break;
+        case SyncResult::Problem: // don't show the problem icon in tray.
+        case SyncResult::Success:
+            if( overallResult.status() == SyncResult::Undefined )
+                overallResult.setStatus( SyncResult::Success );
+            break;
+        case SyncResult::Error:
+            overallResult.setStatus( SyncResult::Error );
+            break;
+        case SyncResult::SetupError:
+            if ( overallResult.status() != SyncResult::Error )
+                overallResult.setStatus( SyncResult::SetupError );
+            break;
+        case SyncResult::SyncAbortRequested:
+            break;
+            // no default case on purpose, check compiler warnings
+        }
+    }
+    return overallResult;
 }
+
+QString FolderMan::statusToString( SyncResult syncStatus, bool enabled ) const
+{
+    QString folderMessage;
+    switch( syncStatus.status() ) {
+    case SyncResult::Undefined:
+        folderMessage = tr( "Undefined State." );
+        break;
+    case SyncResult::NotYetStarted:
+        folderMessage = tr( "Waits to start syncing." );
+        break;
+    case SyncResult::SyncPrepare:
+        folderMessage = tr( "Preparing for sync." );
+        break;
+    case SyncResult::SyncRunning:
+        folderMessage = tr( "Sync is running." );
+        break;
+    case SyncResult::Unavailable:
+        folderMessage = tr( "Server is currently not available." );
+        break;
+    case SyncResult::Success:
+        folderMessage = tr( "Last Sync was successful." );
+        break;
+    case SyncResult::Error:
+        break;
+    case SyncResult::Problem:
+        folderMessage = tr( "Last Sync was successful, but with warnings on individual files.");
+        break;
+    case SyncResult::SetupError:
+        folderMessage = tr( "Setup Error." );
+        break;
+    case SyncResult::SyncAbortRequested:
+        folderMessage = tr( "User Abort." );
+        break;
+    // no default case on purpose, check compiler warnings
+    }
+    if( !enabled ) {
+        // sync is disabled.
+        folderMessage = tr( "%1 (Sync is paused)" ).arg(folderMessage);
+    }
+    return folderMessage;
+}
+
+} // namespace Mirall
