@@ -19,33 +19,32 @@
 #include <QSslCertificate>
 #include <QList>
 
-#include "ui_sslerrordialog.h"
+#include "mirall/account.h"
 
 class QSslError;
 class QSslCertificate;
 
-
 namespace Mirall
 {
 
-class SslErrorDialog : public QDialog, public Ui::sslErrorDialog
+namespace Ui {
+class SslErrorDialog;
+}
+
+class SslDialogErrorHandler : public AbstractSslErrorHandler {
+public:
+    bool handleErrors(QList<QSslError> errors, QList<QSslCertificate> *certs, Account*);
+};
+
+class SslErrorDialog : public QDialog
 {
     Q_OBJECT
 public:
-    explicit SslErrorDialog(QWidget *parent = 0);
-    
-    bool setErrorList( QList<QSslError> errors );
-
+    explicit SslErrorDialog(Account *account, QWidget *parent = 0);
+    ~SslErrorDialog();
+    bool checkFailingCertsKnown( const QList<QSslError> &errors );
     bool trustConnection();
-
-    void setCustomConfigHandle( const QString& );
-
-    QList<QSslCertificate> storedCACerts();
-
-signals:
-    
-public slots:
-    void accept();
+    QList<QSslCertificate> unknownCerts() const { return _unknownCerts; }
 
 private:
     QString styleSheet() const;
@@ -55,6 +54,8 @@ private:
 
     QList<QSslCertificate> _unknownCerts;
     QString                _customConfigHandle;
+    Ui::SslErrorDialog     *_ui;
+    Account                *_account;
 };
 } // end namespace
 
